# Generated by Django 1.10.2 on 2016-12-12 00:58

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='MumbleUser',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('username', models.CharField(max_length=254, unique=True)),
                ('pwhash', models.CharField(max_length=40)),
                ('groups', models.TextField(blank=True, null=True)),
            ],
            options={
                'db_table': 'services_mumbleuser',
            },
        ),
    ]
