{% extends "allianceauth/base-bs5.html" %}
{% load i18n %}

{% block page_title %}
    {% translate "Delete Timer" %}
{% endblock page_title %}

{% block content %}
    <div class="col-lg-12">
        <h1 class="page-header text-center">
            {% translate "Delete Timer" %}
            {% include "timerboard/index_button.html" %}
        </h1>
        <div class="container-fluid">
            <div class="col-md-2 col-md-offset-5">
                <div class="row">
                    <form action="" method="post">
                        {% csrf_token %}
                        <p>{% blocktranslate %}Are you sure you want to delete timer "{{ object }}"?{% endblocktranslate %}</p>
                        <input class="btn btn-danger btn-block" type="submit" value="Confirm">
                    </form>
                </div>
            </div>
        </div>
    </div>
{% endblock content %}
