{% load i18n %}
{% load navactive %}
{% load auth_notifications %}
<li class="nav-item {% navactive request 'notifications:' %}"
    id="menu_item_notifications">
    <a class="nav-link"
        href="{% url 'notifications:list' %}">
        <span class="fa">
            {% with unread_count=request.user|user_unread_notification_count %}
            <i class="fas fa-bell{% if unread_count %} text-danger{% endif %}"></i>
            {% endwith %}
        </span>
        <span class="d-lg-none d-md-inline m-2">
            {% translate "Notifications" %}
        </span>
    </a>
</li>
