{% extends "allianceauth/base-bs5.html" %}
{% load bootstrap %}
{% load i18n %}

{% block page_title %}
{% endblock page_title %}

{% block content %}
    <div class="col-lg-12">
        <h1 class="page-header text-center">
            {% block page_header %}
            {% endblock page_header %}
            {% include "timerboard/index_button.html" %}
        </h1>
        <div class="container-fluid">
            <div class="col-md-4 col-md-offset-4">
                <div class="row">
                    <form id="add-timer-form" class="form-signin" role="form" action="" method="POST">
                        {% csrf_token %}
                        {{ form|bootstrap }}
                        <br>
                        <button class="btn btn-lg btn-primary btn-block" type="submit">
                            {% block submit_button_text %}
                            {% endblock submit_button_text %}
                        </button>
                    </form>
                </div>
            </div>
        </div>
    </div>

{% endblock content %}

{% block extra_javascript %}
{% include 'bundles/jquery-datetimepicker-js.html' %}
{% endblock %}

{% block extra_script %}
$('#id_start').datetimepicker({
setlocale: '{{ LANGUAGE_CODE }}',
{% if NIGHT_MODE %}
theme: 'dark',
{% else %}
theme: 'default',
{% endif %}
mask: true,
format: 'Y-m-d H:i',
minDate: 0
});
{% endblock extra_script %}
