# Generated by Django 1.11.5 on 2017-10-07 03:55

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    replaces = [('services', '0001_initial'), ('services', '0002_auto_20161016_0135'), ('services', '0003_delete_groupcache')]

    initial = True

    dependencies = [
        ('auth', '0008_alter_user_username_max_length'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
    ]
