{% load i18n %}

<li class="nav-item active">
    <a href="{% url 'authentication:add_character' %}" class="nav-link" title="{% translate 'Add Character' %}"><i class="fas fa-plus"></i></span><span class="d-lg-none d-md-inline m-2">{% translate 'Add Character' %}</span></a>
</li>
<li class="nav-item">
    <a href="{% url 'authentication:change_main_character' %}" class="nav-link" title="{% translate 'Change Main' %}"><i class="fas fa-random"></i></span><span class="d-lg-none d-md-inline m-2">{% translate 'Change Main' %}</span></a>
</li>
