{% extends "services/services_ctrl_base.html" %}
{% load i18n %}

{% block title %}
    {% translate "Discourse" %}
{% endblock %}

{% block url %}
    <a href="{{ DISCOURSE_URL }}">{{ DISCOURSE_URL }}</a>
{% endblock %}

{% block user %}
    Username: {{ char.character_name }}
{% endblock %}

{% block controls %}
    <a title="Go To Forums" href="{{ DISCOURSE_URL }}" class="btn btn-success">
        <span class="fas fa-arrow-right fa-fw"></span>
    </a>
{% endblock %}
