# Generated by Django 4.0.10 on 2023-07-16 11:41

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('menu', '0003_menuitem_menu_menuit_rank_e880ab_idx'),
    ]

    operations = [
        migrations.AlterField(
            model_name='menuitem',
            name='hide',
            field=models.BooleanField(default=False, help_text='Hide this menu item. If this item is a header all items under it will be hidden too.'),
        ),
        migrations.AlterField(
            model_name='menuitem',
            name='icon_classes',
            field=models.CharField(blank=True, default=None, help_text='Font Awesome classes to show as icon on menu', max_length=150, null=True),
        ),
        migrations.AlterField(
            model_name='menuitem',
            name='parent',
            field=models.ForeignKey(blank=True, help_text='Parent Header. (Optional)', null=True, on_delete=django.db.models.deletion.SET_NULL, to='menu.menuitem'),
        ),
        migrations.AlterField(
            model_name='menuitem',
            name='rank',
            field=models.IntegerField(default=1000, help_text='Order of the menu. Lowest First.'),
        ),
        migrations.AlterField(
            model_name='menuitem',
            name='text',
            field=models.CharField(blank=True, default=None, help_text='Text to show on menu', max_length=150, null=True),
        ),
    ]
