# Generated by Django 2.2.11 on 2020-05-22 13:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mumble', '0009_set_mumble_dissplay_names'),
    ]

    operations = [
        migrations.AddField(
            model_name='mumbleuser',
            name='certhash',
            field=models.CharField(blank=True, editable=False, help_text='Hash of Mumble client certificate as presented when user authenticates', max_length=254, null=True, verbose_name='Certificate Hash'),
        ),
    ]
