# Generated by Django 4.0.6 on 2022-07-18 17:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('timerboard', '0004_timer_type'),
    ]

    operations = [
        migrations.AlterField(
            model_name='timer',
            name='planet_moon',
            field=models.CharField(blank=True, default='', max_length=254),
        ),
    ]
