# Generated by Django 1.11.5 on 2017-09-28 02:16

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('timerboard', '0002_make_strings_more_stringy'),
    ]

    operations = [
        migrations.AlterField(
            model_name='timer',
            name='eve_character',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='eveonline.EveCharacter'),
        ),
        migrations.AlterField(
            model_name='timer',
            name='user',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL),
        ),
    ]
