# Generated by Django 4.0.2 on 2022-08-28 14:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('menu', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='menuitem',
            name='hook_function',
            field=models.CharField(blank=True, default=None, max_length=500, null=True),
        ),
        migrations.AlterField(
            model_name='menuitem',
            name='icon_classes',
            field=models.CharField(blank=True, default=None, max_length=150, null=True),
        ),
        migrations.AlterField(
            model_name='menuitem',
            name='text',
            field=models.CharField(blank=True, default=None, max_length=150, null=True),
        ),
    ]
