{% extends "allianceauth/base-bs5.html" %}
{% load i18n %}
{% block page_title %}
    {% translate "Corporation Member Data" %}
{% endblock page_title %}
{% block content %}
    <div>
        <h1 class="page-header text-center">{% translate "Corporation Member Data" %}</h1>
        <nav class="navbar navbar-default">
            <div class="container-fluid">
                <ul class="nav navbar-nav">
                    <li class="dropdown">
                        <a href="#" id="dLabel" class="dropdown-toggle" role="button" data-toggle="dropdown" aria-haspopup="false" aria-expanded="false">{% translate "Corporations" %}<span class="caret"></span></a>
                        <ul class="dropdown-menu" role="menu" aria-labelledby="dLabel">
                            {% for corpstat in available %}
                                <li>
                                    <a href="{% url 'corputils:view_corp' corpstat.corp.corporation_id %}">{{ corpstat.corp.corporation_name }}</a>
                                </li>
                            {% endfor %}
                        </ul>
                    </li>
                    {% if perms.corputils.add_corpstats %}
                        <li>
                            <a href="{% url 'corputils:add' %}">{% translate "Add" %}</a>
                        </li>
                    {% endif %}
                </ul>

                <form class="navbar-form navbar-right" role="search" action="{% url 'corputils:search' %}" method="GET">
                    <div class="form-group">
                        <input type="text" class="form-control" name="search_string" placeholder="{% if search_string %}{{ search_string }}{% else %}{% translate 'Search all corporations...' %}{% endif %}">
                    </div>
                </form>
            </div>
        </nav>

        {% block member_data %}
        {% endblock member_data %}
    </div>
{% endblock content %}
