{% extends "allianceauth/base.html" %}
{% load i18n %}


{% block page_title %}
    {% blocktranslate with service_name=view.service_name|title %}Delete {{ service_name }} Account?{% endblocktranslate %}
{% endblock page_title %}

{% block content %}
    <div class="col-lg-12">
        <h1 class="page-header text-center">
            {% blocktranslate with service_name=view.service_name|title %}Delete {{ service_name }} Account?{% endblocktranslate %}
        </h1>
        <div class="container-fluid">
            <div class="col-md-2 col-md-offset-5">
                <div class="row">
                    <form action="" method="post">
                        {% csrf_token %}
                        <p>
                            {% blocktranslate trimmed with service_name=view.service_name|title %}
                            Are you sure you want to delete your {{ service_name }} account {{ object }}?
                            {% endblocktranslate %}
                        </p>
                        <input class="btn btn-danger btn-block" type="submit" value="Confirm">
                    </form>
                </div>
            </div>
        </div>
    </div>
{% endblock content %}
