# Generated by Django 1.10.2 on 2016-10-21 02:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('authentication', '0008_set_state'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='authservicesinfo',
            name='discourse_username',
        ),
        migrations.AddField(
            model_name='authservicesinfo',
            name='discourse_enabled',
            field=models.BooleanField(default=False),
        ),
    ]
