{% load i18n %}
<div class="card text-center m-4" style="min-width: 18rem; min-height: 18rem">
    <div class="card-body">
        <h5 class="card-title">{% block title %}{% endblock title %}</h5>
        <p class="card-text">{% block url %}{% endblock url %}</p>
        <p class="card-text">
            {% block active %}
            <span class="badge bg-success">Active</span>
            {% endblock active %}
        </p>
        <p class="card-text">{% block user %}{% endblock user %}</p>
        {% block extra %}{% endblock extra %}
    </div>
    <div class="card-footer">{% block controls %}{% endblock controls %}</div>
</div>
