# Generated by Django 3.2.8 on 2021-10-17 16:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('analytics', '0004_auto_20211015_0502'),
    ]

    operations = [
        migrations.AlterField(
            model_name='analyticspath',
            name='ignore_path',
            field=models.CharField(default='/example/', help_text='Regex Expression, If matched no Analytics Page View is sent', max_length=254),
        ),
    ]
