# Generated by Django 1.10.1 on 2016-09-05 21:39

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='EveAllianceInfo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('alliance_id', models.CharField(max_length=254)),
                ('alliance_name', models.CharField(max_length=254)),
                ('alliance_ticker', models.CharField(max_length=254)),
                ('executor_corp_id', models.CharField(max_length=254)),
                ('is_blue', models.BooleanField(default=False)),
                ('member_count', models.IntegerField()),
            ],
        ),
        migrations.CreateModel(
            name='EveApiKeyPair',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('api_id', models.CharField(max_length=254)),
                ('api_key', models.CharField(max_length=254)),
                ('error_count', models.PositiveIntegerField(default=0)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='EveCharacter',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('character_id', models.CharField(max_length=254)),
                ('character_name', models.CharField(max_length=254)),
                ('corporation_id', models.CharField(max_length=254)),
                ('corporation_name', models.CharField(max_length=254)),
                ('corporation_ticker', models.CharField(max_length=254)),
                ('alliance_id', models.CharField(max_length=254)),
                ('alliance_name', models.CharField(max_length=254)),
                ('api_id', models.CharField(max_length=254)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='EveCorporationInfo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('corporation_id', models.CharField(max_length=254)),
                ('corporation_name', models.CharField(max_length=254)),
                ('corporation_ticker', models.CharField(max_length=254)),
                ('member_count', models.IntegerField()),
                ('is_blue', models.BooleanField(default=False)),
                ('alliance', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='eveonline.EveAllianceInfo')),
            ],
        ),
    ]
