# Generated by Django 3.1.1 on 2021-01-05 14:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0012_index_additions'),
    ]

    operations = [
        migrations.AddField(
            model_name='evecorporationinfo',
            name='ceo_id',
            field=models.PositiveIntegerField(blank=True, default=None, null=True),
        ),
    ]
