# Generated by Django 1.10 on 2017-10-20 13:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('hrapplications', '0002_choices_for_questions'),
    ]

    operations = [
        migrations.AddField(
            model_name='applicationquestion',
            name='multi_select',
            field=models.BooleanField(default=False),
        ),
    ]
